/**
 * vim: set ts=4 :
 * =============================================================================
 * RCON Whitelist Plugin
 * Restricts the IP addresses from which an RCON connection can be stablished
 *
 * SourceMod (C)2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include <sourcemod>
#include <smrcon>

#define CACHE_ARRAY_SIZE	10
#define DB_NAME				"rcon-addresses"
#define DB_ALLOWED_TABLE	"allowed"

public Plugin myinfo = 
{
	name = "RCON Whitelist",
	author = "fakuivan",
	description = "Control which IP addresses can log into your server's RCON",
	version = "1.0",
	url = "https://forums.alliedmods.net/member.php?u=264797"
};

int gi_addresses[CACHE_ARRAY_SIZE][4];
int gi_nofaddresses;
bool gb_use_hardcoded_array = true;
Handle gh_addresses;				//dynamic array used when CACHE_ARRAY_SIZE is smaller than the actual amount of addresses on the database


public void OnPluginStart()
{
	LoadTranslations("common.phrases");
	LoadTranslations("core.phrases");
	LoadTranslations("rcon.whitelist");
	LoadDBCache();
	RegAdminCmd("sm_rconw_print_allowed", Command_GetAddresses, ADMFLAG_RCON, "Shows a list of the IP Addresses allowed to do rcon.");
	RegAdminCmd("sm_rconw_reload_db", Command_ReloadDB, ADMFLAG_RCON, "Reloads the RCON Whitelist cache.");
}

public Action Command_ReloadDB(int client, int args)
{
	PrintToConsole(client, "[SM] %T", "reloading_cache", client);
	LoadDBCache();
	PrintToConsole(client, "[SM] %T", "done", client);	//why isn't this one on common.phrases? ;-;
	return Plugin_Handled;
}

public Action Command_GetAddresses(int client, int args)
{
	if (gb_use_hardcoded_array)
	{
		PrintToConsole(client, "[SM] %T", "allowed_rcon", client);
		for (int i = 0; i < gi_nofaddresses; i++)
		{
			PrintToConsole(client, "[SM] %i.%i.%i.%i", gi_addresses[i][0], gi_addresses[i][1], gi_addresses[i][2], gi_addresses[i][3]);
		}
	}
	else
	{
		int i_address_buff[4];
		PrintToConsole(client, "[SM] %T", "allowed_rcon", client);
		for (int i = 0; i < gi_nofaddresses; i++)
		{
			GetArrayArray(gh_addresses, i, i_address_buff);
			PrintToConsole(client, "[SM] %i.%i.%i.%i", i_address_buff[0], i_address_buff[1], i_address_buff[2], i_address_buff[3]);
		}
	}
	return Plugin_Handled;
}

public Action SMRCon_OnAuth(int rconid, const char[] address, const char[] password, bool &allow)
{
	char buffer[4][4];
	ExplodeString(address, ".", buffer, 4, 4);
	int i_IP[4]
	i_IP[0] = StringToInt(buffer[0]);
	i_IP[1] = StringToInt(buffer[1]);
	i_IP[2] = StringToInt(buffer[2]);
	i_IP[3] = StringToInt(buffer[3]);
	
	if (gb_use_hardcoded_array)
	{
		for (int i = 0; i < gi_nofaddresses; i++)
		{
			if (i_IP[0] == gi_addresses[i][0] && 
				i_IP[1] == gi_addresses[i][1] && 
				i_IP[2] == gi_addresses[i][2] && 
				i_IP[3] == gi_addresses[i][3])
			{
				PrintToServer("[SM] %T", "allowed_connecting", LANG_SERVER, address);
				return Plugin_Continue;
			}
		}
		PrintToServer("[SM] %T", "disallowed_connecting", LANG_SERVER, address);
		allow = false;
		return Plugin_Changed;
	}
	else
	{
		int i_address_buff[4];
		for (int i = 0; i < gi_nofaddresses; i++)
		{
			GetArrayArray(gh_addresses, i, i_address_buff);
			if (i_IP[0] == i_address_buff[0] && 
				i_IP[1] == i_address_buff[1] && 
				i_IP[2] == i_address_buff[2] && 
				i_IP[3] == i_address_buff[3])
			{
				PrintToServer("[SM] %T", "allowed_connecting", LANG_SERVER, address);
				return Plugin_Continue;
			}
		}
		PrintToServer("[SM] %T", "disallowed_connecting", LANG_SERVER, address);
		allow = false;
		return Plugin_Changed;
	}
}

void LoadDBCache()
{
	char s_error[256];
	Database h_db = SQLite_UseDatabase(DB_NAME, s_error, sizeof(s_error));
	if(h_db == INVALID_HANDLE){
		SetFailState(s_error); }
		
	char s_table[] = DB_ALLOWED_TABLE;
	char s_query[sizeof(s_table) + 14];
	Format(s_query, sizeof(s_query), "SELECT * FROM %s", DB_ALLOWED_TABLE);
	DBResultSet h_query = SQL_Query(h_db, s_query);
	gi_nofaddresses = SQL_GetRowCount(h_query);
	
	if (CACHE_ARRAY_SIZE <= gi_nofaddresses)
	{
		gb_use_hardcoded_array = false;
		if (gh_addresses == INVALID_HANDLE) { gh_addresses = CreateArray(4); } 
		else { CloseHandle(gh_addresses); gh_addresses = CreateArray(4); }
		int i_address_cache[4];
		for (int i = 0; SQL_FetchRow(h_query); i++)		
		{
			i_address_cache[0] = SQL_FetchInt(h_query, 0);
			i_address_cache[1] = SQL_FetchInt(h_query, 1);
			i_address_cache[2] = SQL_FetchInt(h_query, 2);
			i_address_cache[3] = SQL_FetchInt(h_query, 3);
			PushArrayArray(gh_addresses, i_address_cache);
		}
	}
	else
	{
		for (int i = 0; SQL_FetchRow(h_query); i++)
		{
			gb_use_hardcoded_array = true;
			gi_addresses[i][0] = SQL_FetchInt(h_query, 0);
			gi_addresses[i][1] = SQL_FetchInt(h_query, 1);
			gi_addresses[i][2] = SQL_FetchInt(h_query, 2);
			gi_addresses[i][3] = SQL_FetchInt(h_query, 3);
		}
	}
	CloseHandle(h_query);
	CloseHandle(h_db);
}
